@@ -292,9 +292,10 @@ ensure_credentials_sync (GoaProvider         *provider,
     {
       if (error != NULL)
         {
-          g_prefix_error (error, _("Did not find password with username `%s' in credentials"), username);
-          (*error)->domain = GOA_ERROR;
-          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
+          *error = g_error_new (GOA_ERROR,
+                                GOA_ERROR_NOT_AUTHORIZED,
+                                _("Did not find password with username `%s' in credentials"),
+                                username);
         }
       goto out;
     }
@@ -310,6 +311,25 @@ ensure_credentials_sync (GoaProvider         *provider,
                                           server,
                                           cancellable,
                                           error);
+  if (!ret)
+    {
+      if (error != NULL)
+        {
+          g_prefix_error (error,
+                          /* Translators: the first %s is the username
+                           * (eg., debarshi.ray@gmail.com or rishi), and the
+                           * (%s, %d) is the error domain and code.
+                           */
+                          _("Invalid password with username `%s' (%s, %d): "),
+                          username,
+                          g_quark_to_string ((*error)->domain),
+                          (*error)->code);
+          (*error)->domain = GOA_ERROR;
+          (*error)->code = GOA_ERROR_NOT_AUTHORIZED;
+        }
+      goto out;
+    }
+
   if (out_expires_in != NULL)
     *out_expires_in = 0;
 
