@@ -256,6 +256,39 @@ is_deny_node (GoaOAuth2Provider *provider, WebKitDOMNode *node)
   return ret;
 }
 
+static gboolean
+is_identity_node (GoaOAuth2Provider *provider, WebKitDOMHTMLInputElement *element)
+{
+  gboolean ret;
+  gchar *element_type;
+  gchar *name;
+
+  element_type = NULL;
+  name = NULL;
+
+  ret = FALSE;
+
+  /* FIXME: This does not show up in
+   *        webkit_dom_document_get_elements_by_tag_name, but can be
+   *        seen in the inspector. Needs further investigation.
+   */
+
+  g_object_get (element, "type", &element_type, NULL);
+  if (g_strcmp0 (element_type, "email") != 0)
+    goto out;
+
+  name = webkit_dom_html_input_element_get_name (element);
+  if (g_strcmp0 (name, "login") != 0)
+    goto out;
+
+  ret = TRUE;
+
+ out:
+  g_free (element_type);
+  g_free (name);
+  return ret;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean
@@ -369,12 +402,12 @@ show_account (GoaProvider         *provider,
   goa_util_add_row_switch_from_keyfile_with_blurb (left, right, object,
                                                    _("Use for"),
                                                    "chat-disabled",
-                                                   _("Chat"));
+                                                   _("C_hat"));
 
   goa_util_add_row_switch_from_keyfile_with_blurb (left, right, object,
                                                    NULL,
                                                    "documents-disabled",
-                                                   _("Documents"));
+                                                   _("_Documents"));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -417,5 +450,6 @@ goa_windows_live_provider_class_init (GoaWindowsLiveProviderClass *klass)
   oauth2_class->get_authentication_cookie = get_authentication_cookie;
   oauth2_class->get_identity_sync        = get_identity_sync;
   oauth2_class->is_deny_node             = is_deny_node;
+  oauth2_class->is_identity_node         = is_identity_node;
   oauth2_class->add_account_key_values   = add_account_key_values;
 }
